// Copyright (c) 1999-2009  Pavel Rychly

#include <finlib/config.hh>
#include <finlib/consumer.hh>
#include <finlib/fromtof.hh>
#include <iostream>
#include <unistd.h>
#include <cstdlib>
//#include <finlib/message.hh>

using namespace std;


static void usage (const char *progname) {
    cerr << "usage: " << progname << " [-j NUM_OF_FILES] [-b BUFF_SIZE] [-a]"
	 << " [-l ALIGNMENT] DELTAREV_PATH [FILENAME]\n"
	 << "    -j    join phase only\n"
	 << "    -a    append to existing deltarev\n";
}


int main (int argc, char **argv) 
{
    char *outname;
    int buff_size = 1048576;
    int alignmult = 1;
    FILE *input = stdin;

    bool sort_phase = true, append_to = false;
    int num_of_files = 0;
    const char *progname = argv[0];
    //msg.setup (progname);
    {
	// process options
	int c;
	while ((c = getopt (argc, argv, "h?j:b:al:")) != -1)
	    switch (c) {
	    case 'j':
		sort_phase = false; 
		num_of_files = atol (optarg);
		break;
	    case 'b':
		buff_size = atol (optarg);
		break;
	    case 'l':
		alignmult = atol (optarg);
		break;
	    case 'h':
	    case '?':
		usage (progname);
		return 2;
	    case 'a':
		append_to = true;
		break;
	    default:
		cerr << "unknown option (" << c << ")\n";
		usage (progname);
		return 2;
	    }
	
	if (optind < argc) {
	    outname = argv [optind++];
	    if (optind < argc)
		input = fopen (argv [optind], "r");
	} else {
	    usage (progname);
	    return 2;
	}
    }

    //msg.setup (progname, outname);
    //msg ("file: >%s<\nbuff: %i\n", outname, buff_size);

    try {
	if (sort_phase) {
	    RevFileConsumer *drevcons = 
		RevFileConsumer::create (outname, buff_size, alignmult, 
					 append_to);
	    FromFile<int64_t> ci (input);
	    int64_t id, pos;
	    while (!ci) {
		id = *ci;
		++ci;
		pos = *ci;
		++ci;
		drevcons->put (id, pos);
	    }
	    delete drevcons;
	} else {
	    finish_rev_files (outname, num_of_files, alignmult);
	}
    } catch (exception &e) {
	cerr << e.what() << endl;
	return 1;
    }
    return 0;
}
